function V = vfhasbc(theta1,theta2,msel1,msel2)
%computes asymptotic variance for LR test for AS(2007)-type models
%This is a supplementary function for computation of empirical distance (pfhasbc.m)
%at the business cycle frequencies.

%solve both models
[TT1,TC1,TEPS1,TETA1,RC1] = dsgesolv(theta1,msel1);
[TT2,TC2,TEPS2,TETA2,RC2] = dsgesolv(theta2,msel2);

%check whether both model structures have solutions
if isempty(TT1) || isempty(TT2)
    disp('empty solution: nonexistence/numerical issue') 
    V=[];
    return
end

neq1=size(TT1,2); %number of equations - model 1
neq2=size(TT2,2); %number of equations - model 2

%observables selection matrix
A=[1	0	0	0	0	0	0;...
   0	1	0	0	0	0	0;...
   0	0	1	0	0	0	0];

%the integral [-pi,pi] is approximated using n points
n=10000;
omi=2*pi*(-(n/2-1):1:(n/2))'/n; %form vector of Fourier frequencies for approximation of the integral in G

%identify BC frequencies
ind1=find(omi>=-pi/3 & omi<=-pi/16);
ind2=find(omi>=pi/16 & omi<=pi/3);
idbc=[ind1;ind2];

%update to BC frequencies for computation
omi=omi(idbc);
n2=length(omi);

dimy=size(A,1); %number of observables
sp_mat1=zeros(dimy*n2,dimy); %blank for spectra at all frequencies in omi
sp_mat2=sp_mat1;

%compute model 1 spectrum over all frequencies

if RC1==[1;1] %determinacy in model 1
    RR1 = [TEPS1,zeros(neq1,1)];
    QQ1=createcov_as(theta1); %diagonal covariance matrix
    
    elseif RC1==[1;0] %if undeterminacy in model 1
    TETA1=rref(TETA1').'; %reduced column echelon form for TETA
    RR1 = [TEPS1,TETA1];
    QQ1=createcov_as(theta1); %augmented covariance matrix

    else
    V=[];
    disp('Nonexistence or numerical problems in model1')
    return
    
end

for i=1:n2
exe=exp(-1i*omi(i));
mat1=(eye(neq1)-TT1*exe)\eye(neq1); %inv(1-T1L)
mat2=mat1';  %note that ' gives the conjugate transpose


sp_mat1(((i-1)*dimy+1):(i*dimy),:)=A*mat1*RR1*QQ1*RR1'*mat2*A'/(2*pi);

end

   


clear mat1 mat2 exe

%compute model 2 spectrum over all frequencies

if RC2==[1;1] %determinacy in model 1
    RR2 = [TEPS2,zeros(neq2,1)];
    QQ2=createcov_as(theta2); %diagonal covariance matrix
    
    elseif RC2==[1;0] %if undeterminacy in model 1
    TETA2=rref(TETA2').'; %reduced column echelon form for TETA
    RR2 = [TEPS2,TETA2];
    QQ2=createcov_as(theta2); %augmented covariance matrix

    else
    V=[];
    disp('Nonexistence or numerical problems in model2')
    return    
end

    
for i=1:n2
exe=exp(-1i*omi(i));
mat1=(eye(neq2)-TT2*exe)\eye(neq2); %inv(1-T1L)
mat2=mat1';  %note that ' gives the conjugate transpose


sp_mat2(((i-1)*dimy+1):(i*dimy),:)=A*mat1*RR2*QQ2*RR2'*mat2*A'/(2*pi);

end


%approximate the integral
vint=0; %initialize the quantity inside the integral
for i=1:n2
    
    temp = ((eye(dimy) - (sp_mat1(((i-1)*dimy+1):(i*dimy),:)/sp_mat2(((i-1)*dimy+1):(i*dimy),:)))^2); %(I-f_theta*inv(h_phi))
    
    vint = vint + trace(temp); %term inside the integral
    
    %exvint(i,1)=real(vint);
end

V = real((1/(4*pi))*(2*pi/n)*vint); %final result. Take real part since very small complex residual may remain.

end