function X = appll_as_postn(theta0)
%We use this function to compute full spectrum log posterior
%The model is as in AS(2007): tau; r-star instead of
%beta, and standard deviations instead of variances.

%Input is the  parameter vector transformed using invtransparas.m function
% as well as determinacy indicator: 1 for indeterminacy, determinacy
% otherwise.
%In case of indeterminacy, the parameters used are the elements of the bottom row of the lower triangular 
%Cholesky decomposition of the covariance matrix 

%For dynamic parameter estimation we specify w from 1 to T-1
%This likelihood should be minimized to obtain the estimator
%Note that theta0 is 15x1
%parameter order:
%[tau rstar kappa psi1 psi2 rhoR rhog rhoz sigR sigg sigz, m_eta_r, m_eta_g, m_eta_z, sig_eta]';


%back out parameters from transformation
theta0=transparas(theta0);

if length(theta0)==15
    load swdatai %load indeterminate sample
else
    load swdatad %load determinate sample
end


x=[x(:,2),x(:,1),x(:,3)]; %rearrange order of variables

n=length(x);
w=2*pi*(1:1:n-1)'/n; %frequencies for approximate (Whittle) log likelihood
nw=length(w);

para=partrans(theta0,2); %transofrm parameters for model solution
%% solve the model

ny=size(x,2); %number of variables
[TT,TC,TEPS,TETA,RC] = dsgesolv(para,1);
if isempty(TT)
    disp('empty solution: nonexistence/numerical issue') 
    X=1e07;
    return
end
neq=size(TT,2); %number of equations
if RC==[1;1] %determinacy

pervec=zeros(n,ny^2); %blank

for i=1:ny
for j=1:ny
    %pervec(:,j+(i-1)*ny)=periodogram({x(:,i),x(:,j)},rectwin(n),n); %(cross)periodograms across possibe Fourier frequencies in vech from
    pervec(:,j+(i-1)*ny)=(fft(x(:,i)).*conj(fft(x(:,j))))./(2*pi*n);
end
end
pervec=pervec(2:end,:); %get rid of frequency 0

%preparations to compute spectrum
A=[1	0	0	0	0	0	0;...
   0	0	0	0	1	0	0;...
   0	0	1	0	0	0	0];%...
 %   %selection matrix A. 
term1=0;  %initialize terms 1 and 2 for cumulative summation
term2=0;

QQ=createcov_as(theta0); %form covariance matrix
QQ=QQ(1:3,1:3); %select determinate part
%Check covariance matrix is positive definite
[junk,pp]=chol(QQ); %Cholesky decomp.If pp=0, matrix PD; otherwise - penalize likelihood.
if pp~=0
    disp('covariance matrix not PD')
    X=1e07;
    return
end
for i=1:nw; %compute spectrum
exe=exp(-1i*w(i));
A(2,2)=1-exe; %change to use output growth
fact1=A*((eye(neq) - TT*exe)\TEPS);

ftheta=(1/(2*pi))*(fact1*QQ*fact1'); %compute spectrum using representation (4) in the paper

term1=term1+log(det(ftheta)); %first term in the Whittle likelihood

pd=reshape(pervec(i,:),ny,ny).'; %sample periodogram matrix at w(i)

term2=term2+trace(ftheta\pd); %second term in the likelihood

end
WL=real(term1+term2); %small complex residual of order e-15i sometimes remains

lp=logprasn(theta0); %compute minus log prior density
X=0.5*WL+lp; %final answer: minus log posterior



elseif RC==[1;0] %indeterminacy

TETA=rref(TETA').'; %reduced column echelon form for TETA

%sample periodograms  and cross-periodograms
pervec=zeros(n,ny^2); %blank
for i=1:ny
for j=1:ny
    %pervec(:,j+(i-1)*ny)=periodogram({x(:,i),x(:,j)},rectwin(n),n); %(cross)periodograms across possibe Fourier frequencies in vech from
    pervec(:,j+(i-1)*ny)=(fft(x(:,i)).*conj(fft(x(:,j))))./(2*pi*n);
end
end
pervec=pervec(2:end,:); %get rid of frequency 0

%preparations to compute spectrum

A=[1	0	0	0	0	0	0;...
   0	0	0	0	1	0	0;...
   0	0	1	0	0	0	0];%...
 %selection matrix A

term1=0;  %initialize terms 1 and 2 for cumulative summation
term2=0;
RR=[TEPS,TETA];

QQ=createcov_as(theta0);

%Check covariance matrix is positive definite
[junk,pp]=chol(QQ); %Cholesky decomp.If pp=0, matrix PD; otherwise - penalize likelihood.
if pp~=0
    disp('covariance matrix not PD')
    X=1e07;
    return
end

for i=1:nw; %compute spectrum (see gmatrix.m for details)
exe=exp(-1i*w(i));
A(2,2)=1-exe; %change to use output growth

mat1=(eye(neq)-TT*exe)\eye(neq);
mat2=mat1';  %note that ' gives the conjugate transpose

ftheta=A*mat1*RR*QQ*RR'*mat2*A'/(2*pi); %compute spectrum using representation in the paper
term1=term1+log(det(ftheta)); %first term in the Whittle likelihood
pd=reshape(pervec(i,:),ny,ny).'; %sample periodogram matrix at w(i)
term2=term2+trace(ftheta\pd); %second term in the likelihood


end
WL=real(term1+term2); %small complex residual of order e-15i sometimes remains

lp=logprasn(theta0); %compute minus log prior density
X=0.5*WL+lp; %final answer: minus log posterior

else %no equilibrium exists/numerical problems. Penalize likelihood so it doesn't get picked.
    X=1e07; 
end
end