%% Local Identification check for AS(2007) model, Section 8.1.1

%% Setup

n=1000; %number of quadrature points used in computing the integral in G(theta)
nit=13500; %number of steps along the nonidentification curve
h=1e-04; %step size for the Euler approximation of the nonidentification curve
d=0; %indeterminacy indicator; =0 if determinacy,=1 if indeterminacy
%% Indeterminacy

%Compute the local identification condition:

load theta0asd %load the posterior mean



Gi=gmatas(theta,n,d);

[veci vali] = eig(Gi);


disp 'Rank of G(theta0):'
disp(rank(Gi))

disp 'Smallest eigenvalue:'  
disp(min(diag(vali)))

disp 'Default tolerance level:'
disp(size(Gi,1)*eps(norm(Gi)))

%% Nonidentification curve

nt=length(theta);
G=zeros(nt,nt);
theta2=zeros(nit+1,nt);
theta2(1,:)=theta';
rankvec2=zeros(nit+1,1);
ctheta2=zeros(nt,nit+1);
eigcheck2=zeros(nt,nit+1);
theta0=theta;

for k=2:nit+1
    G=gmatas(theta2(k-1,:),n,d);
    rankvec2(k,1)= rank(G); %keep track of rank
    [vect, val] = eig(G);
    b=sort(diag(val));
    a=find(diag(val)==b(1)); %choose smallest eigenvalue
    c0=real(vect(:,a));
    if c0(4,1)>0 %choose direction (>0=Direction 1; <0=Direction 2)
    c0=c0*(-1);
    end
    ctheta2(:,k)=c0;
    eigcheck2(:,k)=diag(val);
    clear c0 a
    theta2(k,:)=theta2(k-1,:)+ h*ctheta2(:,k)';
end

%% Compute deviation mesures and empirical distances

thetad1=theta2(13002,:)'; %norm(theta0-theta)= 0.045 approx.


%load true_spectrum0asi %load benchmark spectrum
msel=1;
w=2*pi*(-(n/2-1):1:(n/2))'/n;

ny=3;
benchmark=specas(theta,msel,w);


ad1=abs(benchmark-specas(thetad1,msel,w));
rd1=ad1./abs(benchmark);
maxdev_a1(1,:)=max(ad1); %maximum absolute deviations


for jj=1:9
    windex1(1,jj)=max(find(ad1(:,jj)==max(ad1(:,jj))));
    maxdev_rr1(1,jj) = rd1(windex1(1,jj),jj); % maximum absolute deviations in relative form

    
end %measure 2 converts maximum abs deviations in maxdev_a1 and converts them into relative

disp 'Maximum absolute deviation on the curve when norm(theta0-theta)=1.30'
disp(max(maxdev_a1))
disp 'Maximum absolute deviation in relative form when norm(theta0-theta)=1.30'
disp(max(maxdev_rr1)) 


    
%Empirical distance measure
T=[80;150;200;1000];
    
for i=1:length(T)
    
    ed1(i)=pfhas(theta,thetad1,msel,msel,0.05,T(i));

end


disp 'Empirical distance measures on the curve when norm(theta0-theta)=1.30'
disp '  T =  80       150      200       1000'
disp(ed1)

save Section8_1_1_d_2