%%%Obtaining results in Table 1 for the AS(2007) model.

%The code should take about 71 minutes to execute on a Xeon E5-2665 8-core
%2.4Ghz processor, using Matlab R2015a (parallel computation not used
%here).
%% Posterior modes under determinacy and indeterminacy
priomean=[2;2;0.5;1.1;0.25;0.5;0.7;0.7;0.31;0.38;0.75;0;0;0;0.25]; %prior mean/starting value 

svalp=invtransparas(priomean); %transform starting value to unconstrained for optimization

%Indeterminacy
np=size(svalp,1); %no of parameters
crit = 1E-10;		% tolerance 
Hinit = eye(np)*1e-4;	% initial Hessian, no information
nit  = 1000; %max number of iterations

%perform optimization using csminwel algorithm
tic;
[pminp,xstarp,pmgradp,pmhessp,itctp,fcountp,retcodep] = csminwel('appll_as_postn',svalp,Hinit,[],crit,nit);
toc;

pmodei=transparas(xstarp); %transform parameters back to constrained
pmodei2=partrans(pmodei,2); %convert rstar to beta for table output

%Determinacy 

svalp=svalp(1:11); %starting parameters
np=size(svalp,1); %no of parameters
crit = 1E-10;		% tolerance 
Hinit = eye(np)*1e-4;	% initial Hessian, no information
nit  = 1000; %max number of iterations

tic;
[pminp,xstarp,pmgradp,pmhessp,itctp,fcountp,retcodep] = csminwel('appll_as_postn',svalp,Hinit,[],crit,nit);
toc;

pmoded=transparas(xstarp); %transform parameters back to constrained
pmoded2=partrans(pmoded,2); %convert rstar to beta for table output

%% Compute Hessians using AS(2007) robust numerical procedure
%for this section we comment out the transformation part in the logposterior

%Indeterminacy

ashess_i=nhess('appll_as_postn2',pmodei);

X=-ashess_i;

[U,S,V] = svd(X,0);
s = diag(S);	
r = rank(X);		% use theoretical rank

% inverse of negative Hessian
S(:) = 0;

    S(1:r,1:r) = diag(ones(r,1)./s(1:r));
	ashessi    = U*S*U';

%Determinacy    

[ashess_d, hesserr]=nhess('appll_as_postn2',pmoded);

X=-ashess_d;

[U,S,V] = svd(X,0);
s = diag(S);
r = rank(X);		% use theoretical rank

% inverse of negative Hessian
S(:) = 0;
    S(1:r,1:r) = diag(ones(r,1)./s(1:r));
	ashessd    = U*S*U';
    
% Save results
save  asmodei pmodei pmodei2 ashessi
save  asmoded pmoded pmoded2 ashessd
%% MCMC FD estimation indeterminacy

clear
load swdatai
x=[x(:,2),x(:,1),x(:,3)]; %rearrange order of variables
load asmodei

pmhess_i=ashessi;
xstar_i=pmodei;
B=50000; %set number of burn-in values
N=100000; %set number of effective draws in the Markov chain
M=N+B; %total number of draws made
c=[0.30]; %scaling constant for covariance matrix (should be calibrated for rejection rate of 75-77%)
clearvars -except B N M c x pmhess_i xstar_i
np=length(xstar_i);
CH=zeros(np,M); %storage for draws

for k=1:length(c)
    sigma=c(k)*pmhess_i; %proposal covariance matrix (inverse of the Hessian times scaling constant)
parstart= xstar_i ; %starting value
CH(:,1)=parstart; %initialize the mean of the proposal distribution;
rejc=zeros(M-1,1); %rejection counter storage
probac=zeros(M-1,1); %probability of acceptance
lik=zeros(M,1); 

pflag=probac; %record probability of acceptance (for diagnostic use only)
lik(1,1)=appll_asn(parstart,x); %initial likelihood value
rng(4662667) %set seed
tic;
for i=2:M
    if mod(i,1000)==0
        i %display counter at each 1000th iteration
    end
    y=mvnrnd(zeros(np,1),sigma,1).' + CH(:,i-1); %draw a parameter vector with previous draw as a mean
    
  
    u=rand(1,1); %generate uniform on (0,1)
    %Compute acceptance probability
    flag=parflagas(y); %flag for violating bounds of parameters
    if flag==0
    lik(i,1)=appll_asn(y,x); %record likelihood at the draw (diagnostic use)
    else
        lik(i,1)=-1e07; %penalize drawing negative parameter
    end
    alpha=exp(lik(i,1) - lik(i-1,1)); %ratio of densities
    
    probac(i-1,1)=alpha; %record probability of accepting (diagnostic use)
    
    if u<=alpha
        CH(:,i)=y; %accept draw
        rejc(i-1,1)=0; %record acceptance of the draw
    else
        CH(:,i)=CH(:,i-1); %keep previous draw
        rejc(i-1,1)=1; %record rejection of current draw
        lik(i,1)=lik(i-1,1); %fill in previous likelihood
    end
end
    toc;
    timemcmci=toc;
end
save mcmc_1500k_i %save draws etc.

%obtain posterior mean and intervals
tabind(:,1)=mean(CH(:,B+1:end)')';
tabind(:,2)=prctile(CH(:,B+1:end),5,2);
tabind(:,3)=prctile(CH(:,B+1:end),95,2);
for i=1:3
tabind(:,i)=partrans(tabind(:,i),2);
end
tabind(2,[2 3])=tabind(2,[3 2]);
%% MCMC FD estimation determinacy
clearvars -except tabind xstar_i timemcmci

load swdatad

x=[x(:,2),x(:,1),x(:,3)]; %rearrange order of variables
load asmoded

pmhess_d=ashessd;
xstar_d=pmoded;
B=50000; %set number of burn-in values
N=100000; %set number of effective draws in the Markov chain
M=N+B; %total number of draws made
c=[0.49]; %scaling constant for covariance matrix (should be calibrated for rejection rate of 75-77%)
clearvars -except tabind xstar_i B N M c x pmhess_d xstar_d
np=length(xstar_d);
CH=zeros(np,M); %storage for draws

for k=1:length(c)
    sigma=c(k)*pmhess_d; %proposal covariance matrix (inverse of the Hessian times scaling constant)
parstart= xstar_d ; %starting value
CH(:,1)=parstart; %initialize the mean of the proposal distribution;
rejc=zeros(M-1,1); %rejection counter storage
probac=zeros(M-1,1); %probability of acceptance
lik=zeros(M,1); 

pflag=probac; %record probability of acceptance (for diagnostic use only)
lik(1,1)=appll_asn(parstart,x); %initial likelihood value
rng(4662667) %set seed
tic;
for i=2:M
    if mod(i,1000)==0
        i %display counter at each 1000th iteration
    end
    y=mvnrnd(zeros(np,1),sigma,1).' + CH(:,i-1); %draw a parameter vector with previous draw as a mean
    
  
    u=rand(1,1); %generate uniform on (0,1)
    %Compute acceptance probability
    flag=parflagas(y); %flag for violating bounds of parameters
    if flag==0
    lik(i,1)=appll_asn(y,x); %record likelihood at the draw (diagnostic use)
    else
        lik(i,1)=-1e07; %penalize drawing negative parameter
    end
    alpha=exp(lik(i,1) - lik(i-1,1)); %ratio of densities
    
    probac(i-1,1)=alpha; %record probability of accepting (diagnostic use)
    
    if u<=alpha
        CH(:,i)=y; %accept draw
        rejc(i-1,1)=0; %record acceptance of the draw
    else
        CH(:,i)=CH(:,i-1); %keep previous draw
        rejc(i-1,1)=1; %record rejection of current draw
        lik(i,1)=lik(i-1,1); %fill in previous likelihood
    end
end
    toc;
    timemcmcd=toc;
end
save mcmc_1500k_d %save draws

%obtain posterior mean and intervals
tabd(:,1)=mean(CH(:,B+1:end)')';
tabd(:,2)=prctile(CH(:,B+1:end),5,2);
tabd(:,3)=prctile(CH(:,B+1:end),95,2);
for i=1:3
tabd(:,i)=partrans(tabd(:,i),2);
end
tabd(2,[2 3])=tabd(2,[3 2]);

%save final results
pmodei=partrans(xstar_i,2);
pmoded=partrans(xstar_d,2);
pmeand=tabd(:,1);
postint90lowd=tabd(:,2);
postint90hid=tabd(:,3);
pmeani=tabind(:,1);
postint90lowi=tabind(:,2);
postint90hii=tabind(:,3);

save mcmc_res pmoded pmeand postint90lowd postint90hid pmodei pmeani postint90lowi postint90hii

%% Display results
%display table columns:
%load mcmc_res to display
tab1=[pmodei,tabind,[pmoded;zeros(4,1)],[tabd;zeros(4,3)]];

%Indeterminacy part
printmat(tab1(:,1:4), 'Table 1 - Indeterminacy', 'tau beta kappa psi1 psi2 rhor rhog rhoz sigr sigg sigz m_eta_r m_eta_g m_eta_z sig_eta', 'Mode Mean 90%_Lo 90%_Hi');

%Determinacy part
printmat(tab1(:,5:8), 'Table 1 - Determinacy', 'tau beta kappa psi1 psi2 rhor rhog rhoz sigr sigg sigz m_eta_r m_eta_g m_eta_z sig_eta', 'Mode Mean 90%_Lo 90%_Hi');