function [c, ceq] = constraintswgbc(xest,ns,wgt,nrm,indp)
%Constraint function for global identification under determinacy (SW (2007) model)
%Keeps the specified between two parameter vectors >= ns (neighborhood size)

%Steady state growth rate, discount factor, and Taylor rule inflation coefficient fixed in optimization.

%Inputs:
%xest - 1x(p+q-3) candidate vector of parameters
%ns - neighborhood size (denoted "c" in the paper)
%wgt - 1x(p+q-3) user-specified vector of weights
%nrm - specified norm, takes inputs 1 (for L-1 norm), 2(for L-2 norm) and Inf (for infinity norm)
%indp - vector of indices for parameters to be constrained, in increasing
%order. Note here the parameter order has to take into account parameters
%fixed in optimization. E.g., since tau, the first parameter, is fixed at
%tis default value in optimization, index 1 will correspond to the next
%free parameter.

load theta0swd; %load benchmark model parameters
xcon=xest(indp);
thetain=[theta(1:14);theta(16:32)]'; %parameters fixed in optimization removed
   c = ns-norm((xcon-thetain(indp))./wgt,nrm);
   ceq = [];
end