function KL = klswn(theta1,theta2,msel1,msel2,n)
%computes KL distance for LS(2004)-type models

%solve both models
[TT1,TC1,TEPS1,RC1] = swsolv(theta1,msel1);
[TT2,TC2,TEPS2,RC2] = swsolv(theta2,msel2);

%check whether both model structures have solutions
if isempty(TT1) || isempty(TT2)
    disp('empty solution: nonexistence/numerical issue') 
    KL=[];
    return
end

neq1=size(TT1,2); %number of equations - model 1
neq2=size(TT2,2); %number of equations - model 2



%the integral [-pi,pi] is approximated using n points

w=2*pi*(-(n/2-1):1:(n/2))'/n; %form vector of Fourier frequencies for approximation of the integral in G

dimy=size(TEPS1,2);

sp_mat1=zeros(dimy*n,dimy); %blank for spectra at all frequencies in omi
sp_mat2=sp_mat1;

%observables selection matrix
matB=eye(neq1);
matA=matB(18:24,:);


%the integral [-pi,pi] is approximated using n points

%compute model 1 spectrum over all frequencies

for i=1:n; %loop computes spectra and stacks them in 1 big matrix
    exe=exp(-1i*w(i));
    
    fact1=matA*((eye(neq1) - TT1*exe)\TEPS1);
sp_mat1(((i-1)*dimy+1):(i*dimy),:)=(1/(2*pi))*(fact1*fact1'); %compute spectrum using representation (4) in the paper


end;

clear matA matB

%compute model 2 spectrum over all frequencies
matB=eye(neq2);
matA=matB(18:24,:);
for i=1:n; %loop computes spectra and stacks them in 1 big matrix
    exe=exp(-1i*w(i));
    
    fact1=matA*((eye(neq2) - TT2*exe)\TEPS2);
sp_mat2(((i-1)*dimy+1):(i*dimy),:)=(1/(2*pi))*(fact1*fact1'); %compute spectrum using representation (4) in the paper


end;

%approximate the integral
%klint=0; %initialize the quantity inside the integral
term1=0;
term2=0;
for i=1:n %changed
    
    %temp = sp_mat2(((i-1)*dimy+1):(i*dimy),:)\sp_mat1(((i-1)*dimy+1):(i*dimy),:); %inv(h_phi)*f_theta
    
    %klint = klint + (trace(temp) - log(det(temp)) - dimy); %term inside the integral
    term1=term1-log(det(sp_mat2(((i-1)*dimy+1):(i*dimy),:)\sp_mat1(((i-1)*dimy+1):(i*dimy),:))); %second term in the KL


    term2=term2+trace(sp_mat2(((i-1)*dimy+1):(i*dimy),:)\sp_mat1(((i-1)*dimy+1):(i*dimy),:)); %second term in the likelihood
end



WL=real(term1+term2); %small complex residual of order e-15i sometimes remains

KL=WL/(2*n)-dimy/2; %final answer: KL


%KL = real((1/(4*pi))*(2*pi/n)*klint); %final result. Take real part since very small complex residual may remain.

end