function X = kloptas_mp3(thetainput,con,wgt,nrm,indp,bc)

%MP version of the kloptas3.m function to compute KL distance between models for distance minimization
%in AS(2007) model.

%con=[pen,c] 

%pen=1 triggers the penalty in case of constraint violation to be evaluated by the function

% c denotes the size of excluded neighborhood (hence determines constraint fucntion)

%all parameters can vary, current inflation rule, indeterminacy.

%wgt,nrm,indp - these inputs are needed for evaluating the constraint
%function should penalty need to be applied. Refer to
%constraintas.m for details.

%bc=1 makes the function use only the business cycle frequencies in
%computation, set 0 for using full spectrum.


warning('off','all');

thetainput=transfmpas2(thetainput,2,1,[[2:4],[6:15]]); %transform parameters back

load theta0asi

theta=mp(theta); %set theta an mp object

theta0=[theta(1),thetainput(1:3),theta(5),thetainput(4:end)]; 



msel=1; %msel for alternative model





%evaluate constraint and apply flat penalty if point unfeasible
if con(1)==1
    conval=constraintastp(thetainput,con(2),wgt,nrm,indp); %
if conval>1e-10
    X=mp('1e10');
    return
end
end



  
%% solve the model
%Specify the monetary policy rule using msel

[TT2,TC2,TEPS2,TETA2,RC2] = dsgesolv_mp(theta0,msel); %this msel changes depending on problem at hand

if isempty(TT2)
    disp('empty solution: nonexistence/numerical issue') 
    X=mp('1e10');
    return
end
neq=size(TT2,2); %number of equations
dck=rcond(mp(eye(neq))-TT2);

if dck<1e-10
    X=mp('1e10');
    return
end

if RC2==[1;1] %determinacy
     retcode = 0;
   RR2 = [TEPS2, mp(zeros(neq,1))];
 
elseif RC2==[1;0] %indeterminacy
     retcode = 1;
     TETA2=frref(TETA2',1e-16).'; %reduced column echelon form for TETA
  RR2 = [TEPS2, TETA2];
else%no equilibrium exists/numerical problems. Penalize likelihood so it doesn't get picked.
    X=mp('1e10');
    return
end

A=mp([eye(3),zeros(3,4)]);  %selection matrix A

QQ2 = createcov_asmp(theta0);
dd=min(eig(QQ2));

if bc==0 %full spectrum
    
load true_spectrum0asimp %load spectrum at theta0 and Gauss-legendre abscissas/weights computed in MP

X=mp('0');
sqi=mp('-1i');
id=mp(eye(neq));
cc=mp('2*pi');
ny=size(A,1);

for i=1:length(x)/2
    exe=exp(sqi*x(i));
    mat1=(id-TT2*exe)\id; %inv(1-T1L)
    mat2=mat1';  %note that ' gives the conjugate transpose
    spec=A*mat1*RR2*QQ2*RR2'*mat2*A'/cc; %spectral density matrix
    temp=spec\true_spectrum(((i-1)*ny+1):i*ny,:);
    X=X+mp('2')*(trace(temp) - log(det(temp))-mp('3'))*wg(i);

end

elseif bc==1 %bc frequencies only (slow code for now)
    load true_spectrum0asibcmp %load spectrum at theta0 and Gauss-legendre abscissas/weights computed in MP
    
    X=mp('0');
    sqi=mp('-1i');
    id=mp(eye(neq));
    cc=mp('2*pi');
    ny=size(A,1);

for i=1:length(x) %here bc frequencies on one side are loaded
    exe=exp(sqi*x(i));
    mat1=(id-TT2*exe)\id; %inv(1-T1L)
    mat2=mat1';  %note that ' gives the conjugate transpose
    spec=A*mat1*RR2*QQ2*RR2'*mat2*A'/cc; %spectral density matrix
    temp=spec\true_spectrum(((i-1)*ny+1):i*ny,:);
    X=X+mp('2')*(trace(temp) - log(det(temp))-mp('3'))*wg(i);

end
end


X=real(X)*mp('1/4/pi');

if dd<0
    X=mp('1e10');
end



end