function X = transfmpas2(theta,dir,detind,ind)
%Transformation of parameters for AS(2007) model from the unconstrained specification to the
%prior specification and back - replicated from
%Lubik-Schorfheide (2004) Gauss code


%Inputs:

%theta - parameter vector, an mp-object

%dir - direction of the transformation: 1 - from initial specification to
%unconstrained; 2 - the other way around

%detind - (in)determinacy indicator, 1 for indeterminacy, 0 for determinacy

%ind - index of parameters being transformed

%parameter order
%[tau rstar kappa psi1 psi2 rhoR rhog rhoz sigR sigg sigz, m_eta_r, m_eta_g, m_eta_z, sig_eta]';

ntheta=length(theta); %no of parameters

% if ntheta==11
%     detind=0; %detect determinacy
% elseif ntheta==15
%     detind=1; %detect indeterminacy
% end

partype=ones(ntheta,1); %parameter type: 1 - constrained as [a,b], e.g. for rho_gz 
%a-1; b=1; 2 - constrained as [o, inf]; 0 - unconstrained
%The transformation below makes new values to lie in [-unf,inf].

if detind==1 %indeterminacy

     %specify lower and upper bounds under indeterminacy
     lb=mp('[0.01 0.9 0.01 0.01  0.01  0.1  0.1  0.1  0.01  0.01 0.01  -3 -3   -3    0.001]');

     ub=mp('[10   0.999   5  0.9 5 0.99 0.99 0.99   3   3    3      3   3  3 3]');

elseif detind==0 %determinacy

    %specify lower and upper bounds under determinacy
    lb=mp('[0.01 0.9 0.01 1.1  0.01  0.1  0.1  0.1  0.01  0.01 0.01]');

    ub=mp('[10   0.999   5  5 5 0.99 0.99 0.99   3   3    3 ]');
end
    


%column 1 -a, column 2 - b, column 3 - c
abc=[lb(ind)',ub(ind)',mp(ones(ntheta,1))];

onep=mp('1');
twop=mp('2');
halfp=mp('0.5');

if dir==1 %transforming into unconstrained form
    
    for i=1:ntheta
    a=abc(i,1);
    b=abc(i,2);
    c=abc(i,3);
    
    if partype(i,1)==1
        cx=twop*(theta(i)-halfp*(a+b))/(b-a);
        theta(i)=(onep/c)*cx/sqrt(onep-cx^twop);
    elseif partype(i,1)==2
        theta(i)= b + (onep/c)*log(theta(i)-a);
    end
    end


elseif dir==2
    
    for i=1:ntheta
    a=abc(i,1);
    b=abc(i,2);
    c=abc(i,3);
    
    if partype(i,1)==1
        theta(i)=((a+b)/twop) + (halfp*(b-a)*c*theta(i))/(sqrt(onep+c^twop*theta(i)^twop));
    elseif partype(i,1)==2
        theta(i)= a + exp(c*(theta(i)-b));
    end
end
    
    
end

X=theta;
end