function X = kloptswcapel_mp(thetainput,bc)

%MP version of the kloptswcapel.m function to compute KL distance between models for distance minimization
%in SW(2007) model.

%con=[pen,c] 

%pen=1 triggers the penalty in case of constraint violation to be evaluated by the function

% c denotes the size of excluded neighborhood (hence determines constraint fucntion)

%all parameters can vary, current inflation rule, indeterminacy.

%wgt,nrm,indp - these inputs are needed for evaluating the constraint
%function should penalty need to be applied. Refer to
%constraintas.m for details.

%bc=1 makes the function use only the business cycle frequencies in
%computation, set 0 for using full spectrum.


warning('off','all');

thetainput=transfmpsw(thetainput,2,[[1:4],[6:34]]); %transform parameters back

load theta0swd %benchmark model parameters

theta=mp(theta);

theta0=[thetainput(1:4),0.99,thetainput(5:end)];

msel=1; %msel for alternative model





  
%% solve the model
%Specify the monetary policy rule using msel

[TT2,TC2,TEPS2,RC2] = swsolv_mp(theta0,msel); %this msel changes depending on problem at hand

if isempty(TT2)
    disp('empty solution: nonexistence/numerical issue') 
    X=mp('1e10');
    return
end
neq=size(TT2,2); %number of equations
dck=rcond(mp(eye(neq))-TT2);

if dck<1e-10
    X=mp('1e10');
    return
end

if RC2==[1;1] %determinacy
     retcode = 0;
   RR2 = [TEPS2, mp(zeros(neq,1))];
 
elseif RC2==[1;0] %indeterminacy
    X=mp('1e10');
    return
else%no equilibrium exists/numerical problems. Penalize likelihood so it doesn't get picked.
    X=mp('1e10');
    return
end

matB=mp(eye(neq));
matA=matB(18:24,:); %observables selection matrix



if bc==0 %full spectrum
    
load true_spectrum0swdmp %load spectrum at theta0 and Gauss-legendre abscissas/weights computed in MP

X=mp('0');
sqi=mp('-1i');
id=mp(eye(neq));
cc=mp('2*pi');
ny=size(matA,1);

for i=1:length(x)/2
    exe=exp(sqi*x(i));
    mat1=(id-TT2*exe)\id; %inv(1-T1L)
    mat2=mat1';  %note that ' gives the conjugate transpose
    spec=matA*mat1*RR2*RR2'*mat2*matA'/cc; %spectral density matrix
    temp=spec\true_spectrum(((i-1)*ny+1):i*ny,:);
    X=X+mp('2')*(trace(temp) - log(det(temp))-mp('7'))*wg(i);

end

elseif bc==1 %bc frequencies only 
    
    load true_spectrum0swdbcmp %load spectrum at theta0 and Gauss-legendre abscissas/weights computed in MP
    
    X=mp('0');
    sqi=mp('-1i');
    id=mp(eye(neq));
    cc=mp('2*pi');
    ny=size(matA,1);
    
    for i=1:length(x)
        exe=exp(sqi*x(i));
        mat1=(id-TT2*exe)\id; %inv(1-T1L)
        mat2=mat1';  %note that ' gives the conjugate transpose
        spec=matA*mat1*RR2*RR2'*mat2*matA'/cc; %spectral density matrix
        temp=spec\true_spectrum(((i-1)*ny+1):i*ny,:);
        X=X+mp('2')*(trace(temp) - log(det(temp))-mp('7'))*wg(i);
        
    end
end


X=real(X)*mp('1/4/pi');

if abs(X)<=1e-10
    X=0; %prevent numerical issues with tiny negative KL in some cases of equivalence
end

if X<0
    X=1e10; %in case of numerical problem 
end



end