function X = transfmpsw(theta,dir,ind)
%Transformation of parameters for AS(2007) model from the unconstrained specification to the
%prior specification and back - replicated from
%Lubik-Schorfheide (2004) Gauss code


%Inputs:

%theta - parameter vector, an mp-object

%dir - direction of the transformation: 1 - from initial specification to
%unconstrained; 2 - the other way around


%ind - index of parameters being transformed


%Parameter order: 
% ['cgy           ';'cmaw          ';'cmap          ';'calfa         ';'czcap         ';
%     'csadjcost     ';'csigma        ';'chabb         ';'cfc           ';'cindw         ';
%     'cprobw        ';'cindp         ';'cprobp        ';'csig          ';'crpi          ';
%     'crdy          ';'cry           ';'crr           ';'crhoa         ';'crhob         ';
%     'crhog         ';'crhoqs        ';'crhoms        ';'crhopinf      ';'crhow         ';
%     'sigma_a       ';'sigma_b       ';'sigma_g       ';'sigma_qs      ';'sigma_m       ';
%     'sigma_pinf    ';'sigma_w       ';'gambar        ';'100(1/cbeta-1)'];

ntheta=length(theta); %no of parameters



partype=ones(ntheta,1); %parameter type: 1 - constrained as [a,b], e.g. for rho_gz 
%a-1; b=1; 2 - constrained as [o, inf]; 0 - unconstrained
%The transformation below makes new values to lie in [-unf,inf].

lb=mp('[0.010;0.010;  0.010;  0.010;0.010;2;0.250;0.0010;1;0.010;0.30;0.010;0.50;0.250;1.1;0.010;0.010;0.50;0.010;0.010;0.010;0.010;0.010;0.010;0.0010;0.010;0.0250;0.010;0.010;0.010;0.010;0.010;0.10;0.010]')';
ub=mp('[2;    0.90;0.90;1;1;15;3;0.99000;3;0.99000;0.95000;0.99000;0.95000;10;3;0.50000;0.50000;0.97500;0.990;0.990;0.990;0.990;0.990;0.990;0.990;3;5;3;3;3;3;3;0.80000;2]')';

    


%column 1 -a, column 2 - b, column 3 - c
abc=[lb(ind)',ub(ind)',mp(ones(ntheta,1))];

onep=mp('1');
twop=mp('2');
halfp=mp('0.5');

if dir==1 %transforming into unconstrained form

for i=1:ntheta
    a=abc(i,1);
    b=abc(i,2);
    c=abc(i,3);
    
    if partype(i,1)==1
        cx=twop*(theta(i)-halfp*(a+b))/(b-a);
        theta(i)=(onep/c)*cx/sqrt(onep-cx^twop);
    elseif partype(i,1)==2
        theta(i)= b + (onep/c)*log(theta(i)-a);
    end

end

elseif dir==2
    for i=1:ntheta
        a=abc(i,1);
        b=abc(i,2);
        c=abc(i,3);
        
        if partype(i,1)==1
            theta(i)=((a+b)/twop) + (halfp*(b-a)*c*theta(i))/(sqrt(onep+c^twop*theta(i)^twop));
        elseif partype(i,1)==2
            theta(i)= a + exp(c*(theta(i)-b));
        end
        
    end
    
    

end

X=theta;
end